import sys
import types
import traceback
import unittest


class TestRunner(unittest.TextTestRunner):

    def __init__(self, stream=sys.stdout):
        self.stream = unittest._WritelnDecorator(stream)

    def _makeResult(self):
        return TextTestResult(stream=self.stream)


class TextTestResult(unittest.TestResult):

    separator2 = '=' * 70

    def __init__(self, stream):
        unittest.TestResult.__init__(self)
        self.stream = stream

    def printErrors(self):
        self.printErrorList(self.errors)
        self.printErrorList(self.failures)

    def printErrorList(self, errors):
        for test, error in errors:
            self.stream.writeln(error)

    def _exc_info_to_string(self, err, test):
        """Converts a sys.exc_info()-style tuple of values into a string."""
        exctype, value, tb = err
        # Skip test runner traceback levels
        while tb and self._is_relevant_tb_level(tb):
            tb = tb.tb_next

        f = tb.tb_frame
        lineno = tb.tb_lineno
        co = f.f_code
        filename = co.co_filename
        return '%s:%s:%s' % (filename, lineno, format_exception_only(exctype,
                                                                     value))


# from unittest.py

class TestProgram(unittest.TestProgram):

    USAGE = """\
Usage: %(progName)s [options] [test] [...]

Options:
  -h, --help        Show this message
  -v, --verbose     Verbose output
  -q, --quiet       Minimal output
  -g, --gnu-format-error
                    Format error as described in GNU Coding Standards

Examples:
  %(progName)s                               - run default set of tests
  %(progName)s MyTestSuite                   - run suite 'MyTestSuite'
  %(progName)s MyTestCase.testSomething      - run MyTestCase.testSomething
  %(progName)s MyTestCase                    - run all 'test*' test methods
                                               in MyTestCase
"""

    def parseArgs(self, argv):
        import getopt
        try:
            options, args = getopt.getopt(argv[1:], 'hHvqg',
                                          ['help', 'verbose', 'quiet',
                                           'gnu-format-error'])
            for opt, value in options:
                if opt in ('-h', '-H', '--help'):
                    self.usageExit()
                if opt in ('-q', '--quiet'):
                    self.verbosity = 0
                if opt in ('-v', '--verbose'):
                    self.verbosity = 2
                if opt in ('-g', '--gnu-format-error'):
                    self.testRunner = TestRunner()
            if len(args) == 0 and self.defaultTest is None:
                self.test = self.testLoader.loadTestsFromModule(self.module)
                return
            if len(args) > 0:
                self.testNames = args
            else:
                self.testNames = (self.defaultTest,)
            self.createTests()
        except getopt.error, msg:
            self.usageExit(msg)


main = TestProgram


# taken from traceback module

def format_exception_only(etype, value):
    """Format the exception part of a traceback.

    The arguments are the exception type and value such as given by
    sys.last_type and sys.last_value.
    """

    # An instance should not have a meaningful value parameter, but
    # sometimes does, particularly for string exceptions, such as
    # >>> raise string1, string2  # deprecated
    #
    # Clear these out first because issubtype(string1, SyntaxError)
    # would throw another exception and mask the original problem.
    if (isinstance(etype, BaseException) or
        isinstance(etype, types.InstanceType) or
        etype is None or type(etype) is str):
        return _format_final_exc_line(etype, value)

    stype = etype.__name__

    if not issubclass(etype, SyntaxError):
        return _format_final_exc_line(stype, value)

    # It was a syntax error; show exactly where the problem was found.
    try:
        msg, (filename, lineno, offset, badline) = value
    except Exception:
        pass
    else:
        lines = []
        if offset is not None:
            lines.append('%s:' % offset)
        value = msg

    lines.append(_format_final_exc_line(stype, value))
    return ''.join(lines)


def _format_final_exc_line(etype, value):
    """Return a list of a single line --
    normal case for format_exception_only"""
    valuestr = traceback._some_str(value)
    if value is None or not valuestr:
        line = ' %s' % etype
    else:
        line = " %s: %s" % (etype, valuestr)
    return line
